/* nhdctrlcmd.c */
/* version 1.0  of November 2018

   syntax

   nhdctrlcmd TO_NAME DEVICE COMMAND

where 
 
      TO_NAME   is the recipient (remote device receiving the command)
      DEVICE    is the name of the device on TO_NAME for now {OUT1, OUT2, OUT3 ...}
      COMMAND   is the name of the command 
 

the program sends the following command on UDP to the broadcast

nhdctrl command FROM_NAME TO_NAME DEVICE COMMAND TIME-TICKS AUTH

where FROM_NAME is the name of the master sending the command
      TO_NAME   is the recipient (remote device receiving the command)
      DEVICE    is the name of the device on TO_NAME for now {ALL, OUT1, OUT2, OUT3 ...}
      COMMAND   is the name of the command {ON OFF PING SHUTDOWN REBOOT}
      TIME      is the time stamp for this command in hexadecimal (time() 8 characters)
      TICKS     is the output of clock() in hexadecimal 8 characters
      AUTH      is an authorization code (intended to prevent spoofing) 4 characters


For the initial switched devices, the command can be
	ON       (sets output to ON, then returns state in STATE)
	OFF      (sets output 1o OFF, then returns state in STATE)
	PING     (does not change output, then returns state in STATE)
	SHUTDOWN (shutdown the server process)
	REBOOT   (reboot the remote machine)

*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#include "nhdhash.h"
#include "nhdctrlcmd.h"
#include "nhd_socket.h"

#define S_CMD "nhdctrl"

void nhd_initialize(int argc, char **argv, NHD *nhd);


int main(int argc, char **argv)
{
 int flag, i,j,done;
 NHD nhd;
 int snd, buflen, rcv;
 char buf[NHD_BUF_LEN];
 char f_cmd[NHD_BUF_LEN];
 char f_code[NHD_BUF_LEN];
 char f_state[NHD_BUF_LEN];
 char f_addr[NHD_BUF_LEN];
 char f_time[NHD_BUF_LEN];
 char s_time[NHD_BUF_LEN];
 char s_addr[NHD_BUF_LEN];

 time_t t1, t0;

 /* initialize */

 
 nhd_initialize(argc, argv, &nhd);

 /* form command */

 sprintf(s_addr,"%s-%s-%s-%s",nhd.from_name, nhd.to_name, nhd.device, nhd.command);
 sprintf(s_time,"%s-%s-",nhd.stime,nhd.stick);
 sprintf(nhd.buffer,"%s command %s %s",	S_CMD, s_addr, s_time);

 flag = nhdhash_append(nhd.buffer, NHD_BUF_LEN-1);

 if (flag)
  {
   fprintf(stderr," *** ERROR: error in nhdhash_append (%d)\n",flag);
   exit(EXIT_FAILURE);
  }

 buflen = strlen(nhd.buffer);

 printf("%s (buflen=%d)\n",nhd.buffer,buflen);

 done = 0;

 for (i = 0 ; i < NHD_RETRIES && done == 0 ; i++)
  {

   /*  Send command */
     
   snd = nhd_socket_sendbuf(nhd.buffer);

   /* log the command */ 
    
   if (nhd.out != (FILE *) NULL) fprintf(nhd.out,"%s (%d %d)\n",nhd.buffer,snd,buflen);

   /* record the time */

   if (time(&t0) == -1)
    {
      perror("time()");
      exit(EXIT_FAILURE);
    }

   t1 = t0;
    
   /* listen for legitimate response or timeout (if DEBUG, then print all responses */

   while (difftime(t1,t0) < NHD_TIMEOUT && done == 0)
    {
      
     rcv = nhd_socket_getbuf(buf);

     if (rcv > 0)
      {
       printf("%s\n",buf);
       if (nhd.out != (FILE *) NULL)
        fprintf(nhd.out,"%s",buf);

       /* see if a message with a correct hash was received */

       if (nhdhash_test(buf) == 0) // == 0 for correct hash 
        {
         /* parse into sections */

         // nhdctrl completion_code f_addr state f_time
         // nhdctrl, f_addr, and f_time must match 

         sscanf(buf,"%s %s %s %s %s",f_cmd,f_code, f_addr, f_state,f_time);

         if (strcmp(f_cmd,S_CMD) != 0)
          {
           printf(" *** Bad f_cmd (%s) (%s)\n",f_cmd,S_CMD);
           if (nhd.out != (FILE *) NULL)
            fprintf(nhd.out," - Bad f_cmd %s",ctime(&t1));
          }
         else if (strcmp(f_addr,s_addr) != 0)
          {
           printf(" *** Bad f_addr (%s) (%s)\n",f_addr,s_addr);
           if (nhd.out != (FILE *) NULL)
            fprintf(nhd.out," - Bad f_addr %s",ctime(&t1));
          }
         else if (strcmp(f_time,s_time) != 0)
          {
           printf(" *** Bad f_time (%s) (%s)\n",f_time,s_time);
           if (nhd.out != (FILE *) NULL)
            fprintf(nhd.out," - Bad f_time %s",ctime(&t1));
          }
         else if (strcmp(f_code,"done") != 0 && strcmp(f_code,"OBE") != 0)
          {
           printf(" *** Error Code (%s)\n",f_code);
           if (nhd.out != (FILE *) NULL)
            fprintf(nhd.out," - Error Code %s",ctime(&t1));
          }
         else
          {
           if (nhd.out != (FILE *) NULL)
            fprintf(nhd.out,"%s",ctime(&t1));
           done = 1; 
          }

        }
       else
        {
         printf(" *** Bad AUTH\n");
         if (nhd.out != (FILE *) NULL)
          fprintf(nhd.out," - Bad AUTH %s",ctime(&t1));

        }
           
      } 

     if (time(&t1) == -1)
      {
       perror("time()");
       exit(EXIT_FAILURE);
      }

    }
  
   if (buflen != snd) continue;  /* retry */
   
  }

 if (i == NHD_RETRIES)
  {
   printf(" *** ERROR: No response from %s\n",nhd.to_name);
   if (nhd.out != (FILE *) NULL)
    fprintf(nhd.out," *** ERROR: No response from %s - %s",nhd.to_name,ctime(&t1));

   exit(EXIT_FAILURE);
  }
 
 nhd_socket_close();
 return 0;
}
 
void nhd_initialize(int argc, char **argv, NHD *nhd)
{
  if (argc < 4)
   {
    fprintf(stderr," *** ERROR: Syntax nhdctrlcmd.exe TO_NAME DEVICE COMMAND (argc=%d)\n",argc);
    exit(EXIT_FAILURE); 
   }

  strcpy(nhd->from_name,NHD_NAME);

  nhd->to_name = argv[1];
  if (strlen(nhd->to_name) > 255) nhd->to_name[255] = '\0';  /* truncate */

  nhd->device = argv[2];
  if (strlen(nhd->device) > 255) nhd->device[255] = '\0';

  nhd->command = argv[3];
  if (strlen(nhd->command) > 255) nhd->command[255] = '\0';

  nhd->tick = clock();
  sprintf(nhd->stick,"%08x",nhd->tick);

  if (time(&(nhd->time)) == -1)
   {
    fprintf(stderr," *** ERRROR: host cannot return current calendar time\n");
    exit(EXIT_FAILURE);
   }
  sprintf(nhd->stime,"%08x",nhd->time);

  nhd->sauth[0] ='\0';
  
  nhd->out = fopen(NHD_LOGFILE,"a");
  if (nhd->out == (FILE *) NULL)
   {
     fprintf(stderr," *** WARNING: Unable to open logfile (%s)\n", NHD_LOGFILE);
   }

 nhd_init_socket();
     

}