/* nhdhash */

/* Norbert Doerry
   November 2018

   This routine creates a four digit hex number in string format that corresponds to a hash of a string that is provided to it.  Leading and trailing whitespaces are eliminated before calculating the hash.

hash is calculated by using the characters in the string as in index for a table of random numbers.  The random numbers are exclusive or'ed together to form the hash.


The routine returns 0 if successful, 1 if not 

*/

#include <ctype.h>
#include <stdio.h>
#include <string.h>

#include "nhdhash.h"

#define DEBUG 0

/* in_str can be any length 
   out_str must be at least 5 characters (4 digits and '\0'
   If hash_table is not NULL, it must point to a table of 256 random numbers between 0 and 2^16-1
*/


int nhdhash(char *in_str, char *out_str, unsigned int *hash_table)
{
 if (DEBUG){printf(" Entering nhdhash() in_str = (%s)\n",in_str); fflush(stdout);}

 unsigned int *hash, result, ch;
 extern unsigned int default_hash_table[];
 char *s,*ss;
 int i;

 if (in_str == (char *) NULL || out_str == (char *) NULL) return 1;

 if (hash_table != (unsigned int *) NULL)
  hash = hash_table;
 else 
  hash = default_hash_table;

 /* make out_str an empty string */

 out_str[0] ='\0';

 /* skip leading white space */

 for (s = in_str; *s != '\0' && isspace((int) *s) ; s++); 

 if (DEBUG){printf("skip leading spaces - s = (%s)\n",s); fflush(stdout);}

 if (*s == '\0') return 2;  /* no characters to hash */

/* find the end and work backwards (and make sure character is between 0 and 255 */

 ss = s + strlen(s) - 1;

 for ( ; isspace((int) *ss) && ss != s; ss--);
 
 /* *ss now points to last non-space character */

 ss++; 

 *ss = '\0';

 /* s is now a string with no leading or trailing whitespace */

 if (DEBUG){printf("s = (%s)\n",s); fflush(stdout);}

 result = 0;

 /* calculate the hash */

 for (ss = s , i = *s ; *ss != '\0'; ss++ , i++)
  {
    ch = (*ss + 17*i) & 0xff;  /* move through the table */
    result ^= hash[ch];
  }

 result = result & 0xffff;   /* ensure its truncated to 4 bytes */

 sprintf(out_str,"%04x",result);

 if (DEBUG){printf(" leaving nhdhash() out_str = (%s)\n",out_str); fflush(stdout);}

 return 0;

}

// nhdhash_append adds 4 hexidecimal characters to the end of a string if there is sufficient room
// s is the string,  n is the number of characters in the array (including the terminating
// '\0' 
// returns 0 is successful, > 0 if not


int nhdhash_append(char *s, int n)
{
  int slen;
  char hash_string[5];
  int flag;

  if (s == (char *) NULL || n < 5) return 1;  

  slen = strlen(s);

  if (slen + 5 > n)
   return 2;
  
  flag = nhdhash(s, hash_string, (unsigned int *) NULL);
  if (flag)
   return 10 + flag;

  strcat(s,hash_string);

  return 0;
}


/* splits off the last 4 digits of s, hashes the remainder, and sees if the hash matches the last four digits.

returns 0 if there is a match

returns nonzero if not a match or other error 

*/

int nhdhash_test(char *s)
{
 char hash_string[5],hstring[5];
 char *ss;
 int i;
 int flag;

 if (s == (char *) NULL) return 1;

 if (DEBUG) printf("entering nhdhash_test(%s) strlen = (%d)\n",s,strlen(s));

 if (strlen(s) < 5) return 2;  // too short

 for (ss = s+strlen(s) ; isxdigit(*ss) == 0 && ss != s; ss--);  /* find first non xdigit */

 if (ss - s < 4) return 3;  // nothing to hash 

 hash_string[4] = '\0';

 flag = 0;

 for (i=3 ; i >= 0 ; i--,ss--)
  {
    hash_string[i] = *ss;
    if (isxdigit(*ss) == 0) flag = 1;
  }


 ss++;
 *ss = '\0';  // truncate the string at the start of the hash 

 if (DEBUG) { printf("s = (%s)  hash_string = (%s) flag = %d\n",s,hash_string,flag); fflush(stdout);}
 
 if (flag)
  return 4;

 
 flag = nhdhash(s, hstring, (unsigned int *) NULL);

 if (DEBUG) {printf("s = (%s) hstring = (%s) flag = %d\n",s,hstring,flag); fflush(stdout);}

 if (flag)
  return 10+flag;

 flag = strcmp(hstring,hash_string);

 if (DEBUG) {printf("hstring = (%s)  hash_string = (%s) flag =%d\n",hstring,hash_string,flag); fflush(stdout);}

 if (flag)
  return 5;

 return 0;  

}